"""HTML Album module

    begin                : Sat Dec 14 2002
    copyright            : (C) 2002 by Ricardo Niederberger Cabral
    email                : nieder|at|mail.ru
    Time-stamp:            <03/03/09 01:50:51 rnc>
"""

## ***************************************************************************
## *                                                                         *
## *   This program is free software; you can redistribute it and/or modify  *
## *   it under the terms of the GNU General Public License as published by  *
## *   the Free Software Foundation; either version 2 of the License, or     *
## *   (at your option) any later version.                                   *
## *                                                                         *
## ***************************************************************************

import os.path,shutil,sys,traceback,time,os
try:
    import Settings
    from qt import *
except:
    traceback.print_exc()
    print "Error importing the necessary modules."

class HTMLAlbum:
    def __init__(self,WizDlg):
        self.curdb=Settings.curdb
        self.wiz=WizDlg                 # wizard dialog to extract parameters from
        self.styleflist={"Acqua":[      # TODO: a simple style with no gfx frames
                       'aq.css',
                       'aq_background.png',
                       'coininfd.png',
                       'coininfg.png',
                       'coinsupd.png',
                       'coinsupg.png',
                       'd.png',
                       'fond.png',
                       'g.png',
                       'inf.png',
                       'space15_15.png',
                       'uparrow.png',
                       'leftarrow.png',
                       'rightarrow.png',                       
                       'sup.png']
                         }
        self.stylename="Acqua"          # failsafe style
        self.destdir=None               # path to place root index.html or all files if not using the inplace mode
        self.symmode=0                  # if should place everything on destdir and make symlinks to this dir
    def start(self):
        self.symmode=self.wiz.symlinkradio.isChecked()
        try:
            self.destdir=os.path.expanduser(str(self.wiz.destdirEdit.text()))
            if not os.path.exists(self.destdir):
                return "Invalid album destination directory."
            if not os.path.isdir(self.destdir):
                return "Album destination is not a directory."        
            if self.destdir[-1] != '/': self.destdir=self.destdir+'/'
        except:
            return "Error opening destination directory supplied"
        pagename=self.destdir+"index.html"
        self.serverroot=str(self.wiz.serverRootEdit.text())
        if not len(self.serverroot): self.serverroot='/'
        if self.serverroot[-1] != '/':self.serverroot=self.serverroot+'/'
        pagenamew=self.serverroot+"index.html"
        self.indexfile=open(pagename,"w")
        self.header(self.indexfile)
        self.section(self.indexfile,str(self.wiz.titleEdit.text()))
        orphanImgs=[]                   # images that belong to this root page (will happen when the root page is a db dir for example)
        for it in self.curdb.batches[self.wiz.curBatch][1]:            
            txt=""
            if it[0]=="SysDir":         #add sys dir            
                path=it[1]
                htname=self.pagePath(path, self.textParentFolder(pagenamew))
                self.framebig(self.indexfile,"<a href=\""+htname+"\">"+path+"</a>")
            if it[0]=="File" or it[0]=="Img":           #orphan img
                orphanImgs.append(it[1])
            if it[0]=="Dir":            #add dir
                txt=txt+"Images: %d<br>" % len(self.curdb.dirs[it[1]][3])
                txt=txt+"Subfolders: %d<br>" % len(self.curdb.dirs[it[1]][2])                
                path=it[1]
                htname=self.pageDir(path,self.textParentFolder(pagenamew))
                self.framebig(self.indexfile,"<a href=\""+htname+"\">"+self.curdb.dirs[path][0]+"</a>",txt)                
            if it[0]=="Group":
                txt="%s<br>" % self.curdb.groups[it[1]][1]
                txt=txt+"Images: %d<br>" % len(self.curdb.groups[it[1]][3])
                txt=txt+"Subgroups: %d<br>" % len(self.curdb.groups[it[1]][4])                
                path=it[1]
                htname=self.pageGroup(path,self.textParentFolder(pagenamew))
                self.framebig(self.indexfile,"<a href=\""+htname+"\">"+self.curdb.groups[path][0]+"</a>",txt)                
            if it[0]=="Volume": 
                txt="%s<br>" % self.curdb.volumes[it[1]][2]
                txt=txt+"Directories: %d<br>" % len(self.curdb.volumes[it[1]][0])
                path=it[1]
                htname=self.pageVolume(path,self.textParentFolder(pagenamew))
                self.framebig(self.indexfile,"<a href=\""+htname+"\">"+self.curdb.volumes[path][3]+"</a>",txt)
            if it[0]=="SimGroup": #add simgroup
                txt="Similar image(s): %d" % len(self.curdb.simgroups[it[1]][2])
                path=it[1]
                htname=self.pageSimGroup(path,self.textParentFolder(pagenamew))
                #save thumbnail to a picture representing this simgroup
                filen=os.path.split(self.curdb.img[self.curdb.simgroups[it[1]][1]][0])[-1]
                nfilen=self.curdb.img[self.curdb.simgroups[it[1]][1]][0]         # if on inplace mode, this will be replaced by a hased filename, which will be a copy of the respective image
                filenhs="."+self.curdb.hashStr(nfilen)+".png"
                pagepath=self.destdir                
                if not os.path.exists(pagepath+filenhs):
                    try:
                        self.curdb.getThumb(nfilen).save(pagepath+filenhs,"PNG")
                    except:
                        traceback.print_exc()
                        print "Unable to save thumbnail for "+ nfilen
                self.framebig(self.indexfile,"<a href=\""+htname+"\">"+self.getframetext("<img border=0 src=\""+filenhs+"\">")+"</a>",txt)
        if orphanImgs:                  # has orphan images to show
            self.showImgList(self.indexfile,orphanImgs,self.destdir)
        self.footer(self.indexfile)
        self.indexfile.close()
        self.cpStyle(self.destdir)
        return "Album generated."
    def cpStyle(self,path):
        # copy current style files to dir path
        basestyledir=Settings.imgseekdatadir+"styles/"+self.stylename+"/"
        try:
            for fil in self.styleflist[self.stylename]:
                shutil.copyfile(basestyledir+fil,path+"."+fil)
        except:
            traceback.print_exc()
            print "Error copying style files to destination directory."
    def showImgList(self,f,imagelist,pagepath):
        imglist=[] # is a list of lists like this:
        # [href,imgsrc,caption]        
        for i in range(len(imagelist)):
            #sve thumbnail
            imgid=-1
            if type(imagelist[i])==int:
                imgid=imagelist[i]
                imagelist[i]=self.curdb.img[imgid][0]
            filen=os.path.split(imagelist[i])[-1]
            nfilen=imagelist[i]         # if on inplace mode, this will be replaced by a hased filename, which will be a copy of the respective image
            filenhs="."+self.curdb.hashStr(nfilen)+".png"
            if not os.path.exists(pagepath+filenhs):
                try:
                    self.curdb.getThumb(nfilen).save(pagepath+filenhs,"PNG")
                except:
                    print "Unable to save thumbnail for "+ filen
            if self.symmode:
                #filenhs[1:-3]      # hashed filename, this image will be placed on destdir with this name and referenced by this filename
                filen=self.curdb.hashStr(nfilen)+"."+nfilen[-3:]
                if not os.path.exists(pagepath+filen):                                
                    if self.wiz.symlinkcheck.isChecked(): #symlink files
                        try:
                            os.symlink(nfilen, pagepath+filen)
                        except:
                            print "Error creating link:"
                            traceback.print_exc()
                            
                    else: # just copy :-/  diskspace waste
                        try:
                            shutil.copyfile(nfilen,pagepath+filen)
                        except:
                            print "Error copying files:"
                            traceback.print_exc()                            
            # get thumb captions:
            captions="<font size=\"-1\">"
            if imgid!= -1:
                for dridx in range(self.wiz.includedcaptionlist.count()):
                    try:
                        captions=captions+str(self.curdb.meta[imgid][str(self.wiz.includedcaptionlist.text(dridx))])+"<br>"
                    except:
                        traceback.print_exc()                    
                        pass
            else:
                captions=captions+os.path.split(imagelist[i])[-1]+"<br>"
            captions=captions+"</font>"
            imglist.append([filen,filenhs,captions])
        self.showImgs(f,imglist)
        
    def pageDir(self,dirid,header=""):
        """create html page for the given path and return html page name"""
        pagename=None
        path=self.curdb.dirs[dirid][0]
        if self.symmode:
            pagename=self.curdb.hashStr(path)+".html"
            fname=self.destdir+pagename
            pagepath=self.destdir
        else:
            fname=path+"index.html"
            pagepath=path
            pagename=path
        subhtf=open(fname,"w")
        self.header(subhtf)
        if header:
            self.frame(subhtf,header)
        self.section(subhtf,path)
        f=subhtf
        imagelist=[]
        fils=self.curdb.dirs[dirid][4]
        for fil in fils:
            #il=os.path.split(fil)[-1]
            imagelist.append(fil)
        subs=[]
        if self.wiz.recursecheck.isChecked():
            for i in range(len(self.curdb.dirs[dirid][2])):
                ret=self.pageDir(self.curdb.dirs[dirid][2][i],self.textParentFolder("..")) # textParentFolder was "pagename"
                title=self.curdb.dirs[self.curdb.dirs[dirid][2][i]][0]
                txt="Images: %d<br>" % len(self.curdb.dirs[self.curdb.dirs[dirid][2][i]][3])
                txt=txt+"Subfolders: %d<br>" % len(self.curdb.dirs[self.curdb.dirs[dirid][2][i]][2])
                if self.symmode:
                    subs.append([ret,title,txt])
                else:
                    subs.append([self.curdb.dirs[self.curdb.dirs[dirid][2][i]][0],title,txt])
            self.showSubs(subhtf,subs)
        self.showImgList(f,imagelist,pagepath)
        self.footer(subhtf)
        subhtf.close()
        # copy style files to dir
        if not self.symmode: self.cpStyle(path)
        return pagename        

    def pageSimGroup(self,dirid,header=""):
        """create html page for the given path and return html page name"""
        path=self.curdb.img[self.curdb.simgroups[dirid][1]][0]
        pagename=self.curdb.hashStr(path)+".html"
        fname=self.destdir+pagename
        pagepath=self.destdir
        subhtf=open(fname,"w")
        self.header(subhtf)
        if header:
            self.frame(subhtf,header)
        self.section(subhtf,os.path.split(path)[-1])
        f=subhtf
        imagelist=[]
        fils=self.curdb.simgroups[dirid][2]
        for fil in fils:
            #fil=os.path.split(self.curdb.img[fil][0])[-1]
            imagelist.append(self.curdb.img[fil][0])
        subs=[]
        self.showImgList(f,imagelist,pagepath)        
        self.footer(subhtf)
        subhtf.close()
        # copy style files to dir
        if not self.symmode: self.cpStyle(path)
        return pagename        
        
    def pageVolume(self,dirid,header=""):
        """create html page for the given path and return html page name"""
        path=self.curdb.volumes[dirid][3]
        pagename=self.curdb.hashStr(path)+".html"
        fname=self.destdir+pagename
        pagepath=self.destdir
        subhtf=open(fname,"w")
        self.header(subhtf)
        if header:
            self.frame(subhtf,header)
        self.section(subhtf,path)
        f=subhtf
        imagelist=[]
        fils=self.curdb.volumes[dirid][0]
        subs=[]
        for i in range(len(fils)):
            if self.curdb.dirs[fils[i]][1] != -1: continue
            ret=self.pageDir(fils[i],self.textParentFolder(pagename))
            title="%s" % self.curdb.dirs[fils[i]][0]
            txt="Images: %d<br>" % len(self.curdb.dirs[fils[i]][3])
            txt=txt+"Subfolders: %d<br>" % len(self.curdb.dirs[fils[i]][2])                            
            subs.append([ret,title,txt])
        self.showSubs(subhtf,subs)
        self.footer(subhtf)
        subhtf.close()
        # copy style files to dir
        if not self.symmode: self.cpStyle(path)
        return pagename        
        
    def pageGroup(self,dirid,header=""):
        """create html page for the given path and return html page name"""
        path=self.curdb.groups[dirid][0]
        pagename=self.curdb.hashStr(path)+".html"
        fname=self.destdir+pagename
        pagepath=self.destdir
        subhtf=open(fname,"w")
        self.header(subhtf)
        if header:
            self.frame(subhtf,header)
        self.section(subhtf,path)
        f=subhtf
        imagelist=[]
        fils=self.curdb.groups[dirid][3]
        for fil in fils:
            #fil=os.path.split(self.curdb.img[fil][0])[-1]
            imagelist.append(self.curdb.img[fil][0])
        subs=[]
        if self.wiz.recursecheck.isChecked():
            for i in range(len(self.curdb.groups[dirid][4])):
                ret=self.pageGroup(self.curdb.groups[dirid][4][i],self.textParentFolder(pagename))
                tit="%s" % os.path.split(self.curdb.groups[self.curdb.groups[dirid][4][i]][0])[-1]
                txt="<font size=-1>Images: %d<br>" % len(self.curdb.groups[self.curdb.groups[dirid][4][i]][3])
                txt=txt+"Subgroups: %d<br></font>" % len(self.curdb.groups[self.curdb.groups[dirid][4][i]][4])                
                subs.append([ret,tit,txt])
            self.showSubs(subhtf,subs)
        self.showImgList(f,imagelist,pagepath)            
        self.footer(subhtf)
        subhtf.close()
        # copy style files to dir
        if not self.symmode: self.cpStyle(path)
        return pagename        
  
    def pagePath(self,path,header=""):
        """create html page for the given path and return html page name when on symlink mode"""
        if path[-1] != '/': path=path+'/'
        pagename=None        
        if self.symmode:
            pagename=self.curdb.hashStr(path)+".html"
            fname=self.destdir+pagename
            pagepath=self.destdir
        else:
            fname=path+"index.html"
            pagepath=path
            pagename=path
        subhtf=open(fname,"w")
        self.header(subhtf)
        if header:
            self.frame(subhtf,header)
        self.section(subhtf,path)
        f=subhtf
        imagelist=[]
        subdirs=[]
        fils=os.listdir(path)
        for fil in fils:
            if fil[0] == ".": continue
            if os.path.isdir(path+fil):
                subdirs.append(fil)
            else:
                if self.curdb.extIsImg(fil) :
                    imagelist.append(fil)
        subs=[]
        if self.wiz.recursecheck.isChecked():
            for i in range(len(subdirs)):
                ret=self.pagePath(path+subdirs[i],self.textParentFolder(".."))
                if self.symmode:
                    subs.append([ret,subdirs[i],""])
                else:
                    subs.append([subdirs[i],subdirs[i],""])
            self.showSubs(subhtf,subs)
        self.showImgList(f,imagelist,pagepath)            
        self.footer(subhtf)
        subhtf.close()
        # copy style files to dir
        if not self.symmode: self.cpStyle(path)
        return pagename
    def textParentFolder(self,pagename):
        """returns the text to be included when you want a link to the parent folder which is pointed by pagename """
        return "<img border=0 src=\".uparrow.png\"><a href=\""+pagename+"\">Parent folder</a>"
    def showSubs(self,f,subs):
        """write subdirs inside this page """
        # [href,caption,info]
        txt="<h1>Subfolders:</h1>"
        if not subs:return
        for i in subs:
            if i[2]:
                txt=txt+"<img border=0 src=\".rightarrow.png\"><a href=\"" + i[0] + "\">"+i[1]+"</a><br>"+i[2]+"<br>"
            else:
                txt=txt+"<img border=0 src=\".rightarrow.png\"><a href=\"" + i[0] + "\">"+i[1]+"</a><br>"
        self.frame(f,txt)        
    def showImgs(self,f,imglist):
        """ write to f all the images in imglist where imglist is:"""
        # [href,imgsrc,caption]        
        thperrow=4                      # thumnails per row        
        f.write("""<table width=\"100%\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">""")
        for i in range(len(imglist)):
            if i%thperrow==0:
                f.write("</tr>")                        
                f.write("<tr>")
            f.write("<td align=center>")
            self.frame(f,"<center><a href=\""+imglist[i][0]+"\">"+"<img border=0 src=\""+imglist[i][1]+"\"></a><br><font size=\"-1\">"+
                       imglist[i][2]+"</font></center>")
            f.write("</td>")
        f.write("""</table>""")        
    def header(self,f,title="Photo Album"):
        f.write("""
<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">
<html>

<!--	Generated by: imgSeek
	http://imgseek.sourceforge.net/
-->
<head>
    <title>"""+title+"""</title>
    <link rel=\"STYLESHEET\" type=\"text/css\" href=\".aq.css\">
</head>
<body background=\".aq_background.png\" dir=\"ltr\">
""")        
    def footer(self,f):
        f.write("""
<hr>
Generated by <a href=\"http://imgseek.sourceforge.net\">imgSeek</a> - """ +time.asctime(time.localtime()) + """
</body>
</html>        
""")
    def section(self,f,title=""):
        self.frame(f,"""<center><h2>"""+title+"""</h2></center>""")

    def frame(self,f,txt=""):        
        f.write(self.getframetext(txt))
    def getframetext(self,txt=""):
        return """
<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\">
  <tr> 
    <td width=\"15\" height=\"15\" background=\".coinsupg.png\"><img src=\".space15_15.png\" width=\"15\" height=\"15\" alt=\"none\"></td>
    <td height=\"15\" background=\".sup.png\"><img src=\".space15_15.png\" width=\"15\" height=\"15\" alt=\"none\"></td>
    <td width=\"15\" height=\"15\" background=\".coinsupd.png\"><img src=\".space15_15.png\" width=\"15\" height=\"15\" alt=\"none\"></td>
  </tr>
  <tr>
   <td width=\"15\" background=\".g.png\"><img src=\".space15_15.png\" width=\"15\" height=\"15\" alt=\"none\"></td>
   <td background=\".fond.png\" align=\"left\" width=\"100%\">"""+txt+"""</td>
   <td width=\"15\" background=\".d.png\"><img src=\".space15_15.png\" width=\"15\" height=\"15\" alt=\"none\"></td>   
  </tr>
  <tr> 
    <td width=\"15\" height=\"15\" background=\".coininfg.png\"><img src=\".space15_15.png\" width=\"15\" height=\"15\" alt=\"none\"></td>
    <td height=\"15\" background=\".inf.png\"><img src=\".space15_15.png\" width=\"15\" height=\"15\" alt=\"none\"></td>
    <td width=\"15\" height=\"15\" background=\".coininfd.png\"><img src=\".space15_15.png\" width=\"15\" height=\"15\" alt=\"none\"></td>
  </tr>
</table>
"""
    def framebig(self,f,title="",txt=""):
        self.frame(f,"""<h1>"""+title+"</h1><br>"+txt)
