"""IPTC module

    begin                : 03/02/11 21:27:18 rnc
    copyright            : (C) 2002 by Ricardo Niederberger Cabral
    email                : nieder|at|mail.ru
    Time-stamp:            <03/03/16 15:20:56 rnc>

 * Code portions of getiptcinfo function from Fredrik Lundh fredrik@pythonware.com and PIL
""" 
## ***************************************************************************
## *   This program is free software; you can redistribute it and/or modify  *
## *   it under the terms of the GNU General Public License as published by  *
## *   the Free Software Foundation; either version 2 of the License, or     *
## *   (at your option) any later version.                                   *
## *                                                                         *
## ***************************************************************************

try:
    import Image, TiffImagePlugin, JpegImagePlugin, IptcImagePlugin
    import StringIO
except:
    pass

class FakeImage:
    pass

infomap=    {
            5:"Object Name",
			10:"Urgency",
				
			15:"Category",
				
			20:"Supplemental category",
				
			25:"Keywords",
				
			40:"Special instructions",
				
			55:"Creation date",
				
			60:"Creation time",
				
			65:"Originating program",
				
			80:"By Line",
				
			85:"By title",
				
			90:"City",
				
			95:"State/Province",
				
			100:"Country",
				
			101:"Country",
				
			103:"Transref",
				
			105:"Headline",
				
			110:"Credit",
				
			115:"Source",
				
			120:"Caption",
				
			122:"Caption writer"}
				


def getiptcinfo(fname):
    # extract IPTC data from a PhotoShop JPEG or TIFF file
    data = None
    im = Image.open(fname)
    if isinstance(im, JpegImagePlugin.JpegImageFile):
        # extract the IPTC/NAA resource
        try:
            app = im.app["APP13"]
            if app[:14] == "Photoshop 3.0\x00":
                app = app[14:]
                # parse the image resource block
                offset = 0
                while app[offset:offset+4] == "8BIM":
                    offset = offset + 4
                    # resource code
                    code = JpegImagePlugin.i16(app, offset)
                    offset = offset + 2
                    # resource name (usually empty)
                    name_len = ord(app[offset])
                    name = app[offset+1:offset+1+name_len]
                    offset = 1 + offset + name_len
                    if offset & 1:
                        offset = offset + 1
                    # resource data block
                    size = JpegImagePlugin.i32(app, offset)
                    offset = offset + 4
                    if code == 0x0404:
                        # 0x0404 contains IPTC/NAA data
                        data = app[offset:offset+size]
                        break
                    offset = offset + size
                    if offset & 1:
                        offset = offset + 1
        except (AttributeError, KeyError):
            pass

    elif isinstance(im, TiffImagePlugin.TiffImageFile):
        # get raw data from the IPTC/NAA tag (PhotoShop tags the data
        # as 4-byte integers, so we cannot use the get method...)
        try:
            type, data = im.tag.tagdata[TiffImagePlugin.IPTC_NAA_CHUNK]
        except (AttributeError, KeyError):
            pass

    if data is None:
        return None # no properties

    # create an IptcImagePlugin object without initializing it
    im = FakeImage()
    im.__class__ = IptcImagePlugin.IptcImageFile

    # parse the IPTC information chunk
    im.info = {}
    im.fp = StringIO.StringIO(data)

    try:
        im._open()
    except (IndexError, KeyError):
        pass # expected failure

    return im.info
