"""Metadata editor implementation

    begin                : Sat Dec 14 2002
    copyright            : (C) 2002 by Ricardo Niederberger Cabral
    email                : nieder|at|mail.ru
    Time-stamp:            <03/03/09 01:52:17 rnc>
"""
## ***************************************************************************
## *                                                                         *
## *   This program is free software; you can redistribute it and/or modify  *
## *   it under the terms of the GNU General Public License as published by  *
## *   the Free Software Foundation; either version 2 of the License, or     *
## *   (at your option) any later version.                                   *
## *                                                                         *
## ***************************************************************************

from string import *
import Error
try:
    from qt import *
except:
    Error.qtImportError()
try:
    from qttable import QTable,QComboTableItem,QTableItem
except:
    print "Unable to import some QT table modules. Metadata editing may not work"
try:
    from MetaForm import *
    import Settings
except:
    print "MetaWnd: Unable to import some imgSeek modules. Some features will not work."
import traceback

class MetaWnd(metaform):
    def __init__(self, parent,ifile=None): 
        self.curdb=Settings.curdb
        metaform.__init__(self, parent)
        self.imgIdx=0
        self.curfid=None
        #popup 
        self.fieldMenu=QPopupMenu(self) 
        self.fieldMenu.insertItem ( parent.fileNewAction.iconSet(),"&New parameter", self.onNewField)
        self.fieldMenu.insertItem ( parent.removedead.iconSet(), "&Remove parameter", self.onDelField)
        self.fieldMenu.insertSeparator()
        self.fieldMenu.insertItem ( parent.removedead.iconSet(), "Remove &ALL parameters", self.onDelAllField)
        # other inits
        self.metalist=None
        self.combos=[]
        self.selectedRow=None
        self.tableMeta.setColumnStretchable(1,1)
        self.updateCombos()
        self.dirty=0
        self.readonlyfields=self.curdb.readonlyfields
        self.curdb.cbs["Batch"].append(self.syncBatchCombo)                
        if ifile: # requested file to be shown immediately
            self.showFile(ifile)            
        else: #simple show first file on dbase
            self.showFile(self.curdb.meta.keys()[0])
        self.syncBatchCombo()
    def onNewField(self):
        text = str(QInputDialog.getText("imgSeek","New field name:" ,QLineEdit.Normal, "")[0])
        if not text: return
        if text in self.curdb.meta[self.curfid].keys():
            QMessageBox.information( self, "imgSeek","There is already a field with this name, please try another.")
            return
        if text in self.readonlyfields:
            QMessageBox.information( self, "imgSeek","This field is reserved, please try another.")
            return
        self.tableMeta.setNumRows(self.tableMeta.numRows()+1)
        if text not in self.curdb.metafields:
            self.curdb.metafields.append(text)
            self.updateCombos()
        a=QComboTableItem ( self.tableMeta, self.metalist, 1 )
        a.setCurrentItem(text)
        self.combos.append(a)
        self.tableMeta.setItem(self.tableMeta.numRows()-1,0,a)
        self.saveMeta()
        self.showFile(self.curfid)
    def onDelField(self):
        self.tableMeta.removeRow(self.selectedRow)
        self.saveMeta()
    def onDelAllField(self):
        self.curdb.removeAllMeta(self.curfid)
        self.showFile(self.curfid)
    def tableMeta_currentChanged(self,a0,a1):
        return                          # do not save on every single move
        if self.targettab.currentPageIndex() == 0: # image
            self.saveMeta()
    def updateCombos(self):
        """sync all combos to the metafields cache at curdb.metafields """
        if not len(self.curdb.metafields): return        
        lst=QStringList()
        for mtf in self.curdb.metafields:
            lst.append(mtf)
        for cmb in self.combos:
            cmb.setStringList(lst)
        self.metalist=lst
    def showMetaTable(self,tbl):
        self.tableMeta.setNumRows(len(tbl))
        cnt=0
        for cmb in self.combos: del cmb
        self.combos=[]
        kes=tbl.keys()[:]
        kes.sort()
        for field in kes:
            if field not in self.curdb.metafields:
                self.curdb.metafields.append(field)
                self.updateCombos()        
        for field in kes:
            a=QComboTableItem ( self.tableMeta, self.metalist, 1 )
            self.combos.append(a)
            self.tableMeta.setItem(cnt,0,a)
            self.tableMeta.setText(cnt,1,str(tbl[field]))
            a.setCurrentItem(field)
            try:
                if field in self.readonlyfields:
                    self.tableMeta.setItem(cnt,1,QTableItem ( self.tableMeta,QTableItem.Never, str(tbl[field])))
            except:
                traceback.print_exc()
            cnt=cnt+1        
    def showFile(self,fid):
        self.imgIdx=self.curdb.img.keys().index(fid)
        self.curfid=fid
        self.showMetaTable(self.curdb.meta[fid])
        self.thumbLabel.setPixmap(QPixmap(self.curdb.getThumb(self.curdb.img[fid][0])))
        self.indexLabel.setText(str(self.imgIdx+1)+" / "+str(len(self.curdb.img.keys())))
    def saveMeta(self):
        """save metadata for currently shown image. Called by prev/next/apply"""
        self.curdb.meta[self.curfid]={}
        dct=self.metaFromTable()
        for fi in dct.keys():
            self.curdb.meta[self.curfid][fi]=dct[fi]
        self.curdb.dirty=1
        self.dirty=0
    def metaFromTable(self):
        """ returns a meta dict from whats currently on the table"""
        dct={}
        savedfields=[]
        for ri in range(self.tableMeta.numRows()):
            try:
                if str(self.tableMeta.text(ri,0)) in savedfields:                    
                    continue
                savedfields.append(str(self.tableMeta.text(ri,0)))
                dct[str(self.tableMeta.text(ri,0))]=str(self.tableMeta.text(ri,1))
            except:
                print "Error saving metadata field:"+str(ri)
        return dct
    def previousBtn_clicked(self):
        self.saveMeta()        
        self.imgIdx=self.imgIdx-1
        if self.imgIdx<0:self.imgIdx=len(self.curdb.meta.keys()) -1
        self.showFile(self.curdb.meta.keys()[self.imgIdx])
    def nextButton_clicked(self):
        self.saveMeta()
        self.imgIdx=self.imgIdx+1
        if self.imgIdx>=len(self.curdb.meta.keys()):
            self.imgIdx=0
        self.showFile(self.curdb.meta.keys()[self.imgIdx])
    def syncBatchCombo(self,what=None):
        """call it to sync curdb batch list to what the ui shows """
        self.batchcombo.clear()
        for bid in self.curdb.batches.keys():
            self.batchcombo.insertItem(self.curdb.batches[bid][0])
    def ApplyBtn_clicked(self):
        if self.targettab.currentPageIndex() == 0: # image
            self.saveMeta()
        else:                           # batch
            try:
                self.batchmtable=self.metaFromTable()
                bid=self.curdb.batches.keys()[self.batchcombo.currentItem()]
                self.curdb.saveBatchMeta(self.batchmtable,bid)
            except:
                traceback.print_exc()
                print "Error saving setting metadata to batch of images"
    def tableMeta_contextMenuRequested(self,a0,a1,a2):
        self.selectedRow=a0
        self.fieldMenu.popup(a2)
    def targettab_selected(self,a0):
        pass
    def targettab_currentChanged(self,a0):
        if self.targettab.currentPageIndex() == 0: # image
            self.thumbLabel.show()
            self.batchmtable=self.metaFromTable()
            self.showFile(self.curfid)
        else:                           # batch
            self.saveMeta()
            self.thumbLabel.hide()            
            self.showMetaTable(self.batchmtable)
    def showbatcheditor_clicked(self):
        wnd=self.parent()
        wnd.editcategaction_activated()
    def batchcombo_activated(self,a0):
        pass
    def refreshbtn_clicked(self):
        res=QMessageBox.information( self, "imgSeek","Are you sure you want to revert to saved metadata and lose changes you made to the current image ?",   QString("&Revert to what is saved"), QString("Cancel"))
        if res: return                    
        self.showFile(self.curfid)
    def closeEvent(self,e):
        if not self.dirty:
            metaform.closeEvent(self,e)
            return
        res=QMessageBox.information( self, "imgSeek","Do you want to save changes made to current image ?",   QString("&Save"), QString("&Discard"))
        metaform.closeEvent(self,e)
        if res: return
        self.saveMeta()
    def tableMeta_valueChanged(self,a0,a1):
        self.dirty=1
        
