"""Metadata editor implementation

    begin                : Sat Dec 14 2002
    copyright            : (C) 2002 by Ricardo Niederberger Cabral
    email                : nieder|at|mail.ru
    Time-stamp:            <03/03/09 11:29:27 rnc>
"""
## ***************************************************************************
## *                                                                         *
## *   This program is free software; you can redistribute it and/or modify  *
## *   it under the terms of the GNU General Public License as published by  *
## *   the Free Software Foundation; either version 2 of the License, or     *
## *   (at your option) any later version.                                   *
## *                                                                         *
## ***************************************************************************

from string import *
import traceback
import Error
try:
    import ImageFont
    import ImageEnhance
    import ImageDraw
    import ImageChops
    import Image
    import ImageFilter
except:
    traceback.print_exc()
    print "Some Python Imaging Lib modules are missing. Please install PIL correctly if you want the related features."
try:
    from qt import *
except:
    Error.qtImportError()
try:
    import Settings
    from TransformForm import *
    from SlideShowDialog import *
except:
    print "Unable to import some imgSeek modules. Some features will not work."
try:
    import imgdb
except:
    traceback.print_exc()
    print "Warning: Unable to load the C++ extension \"imgdb.so\" module. Make sure you installed imgSeek correctly, and email any install bug to \"imgseek-devel@lists.sourceforge.net\"."


class Transforms:
    """Implements all transforms
    
    for each new transform:
    list it and it's parameters on getTransforms
    create its function:
    b is an unused parameter right now. (just make it 0). a0..n are parameters passed by GUI
    real is a string where it should save changes to. If real is None, then save changes to  .temp thumbnail

    For more complex transforms (use caption as an example), you may want to get gui data from self.wnd instead and simply ignore parameters
    """
    def __init__(self, wnd):            
        self.wnd=wnd
        self.tPixmap=QPixmap()               # temp pixmap holding transformed images
        self.callback=None              # will hold a funtion that should be called whenever a slider changes. arguments to a callback function are (parm index,value)
        self.curdb=self.wnd.curdb
    def getTransforms(self):
        """call to get existing transforms

        @return: list of transforms: [name,callback,[parameter1 type, parameter 1 caption]]
        @rtype: list
        """
        ### Please keep this list alphabetically sorted, or else I'll have to write code to sort it.
        return [
            ["Blur",self.blur,[]],
            ["Caption",self.caption,[["Text","Caption"],["Combo","Position",["Top Left","Top Right","Bottom Left","Bottom Right"]],["Combo","Color",["Black","White","Red","Green","Blue"]]]],
            ["Color",self.color,[["Int","Factor"]]],
            ["Contrast",self.contrast,[["Int","Factor"]]],
            ["Contour",self.CONTOUR,[]],
            ["Darken",self.bright,[["Int","Factor"]]],
            ["Detail",self.DETAIL,[]],
            ["Edge enhance",self.EDGE_ENHANCE,[]],
            ["Edge enhance more",self.EDGE_ENHANCE_MORE,[]],
            ["Emboss",self.EMBOSS,[]],
            ["Find edges",self.FIND_EDGES,[]],
            ["Invert",self.invert,[]],            
            ["Rotate",self.rotate,[["Int","Degree"]]],            
            ["Sharpen",self.SHARPEN,[]],            
            ["Sharpness",self.sharp,[["Int","Factor"]]],            
            ["Smooth",self.SMOOTH,[]],
            ["Smooth more",self.SMOOTH_MORE,[]]
            ]
    def caption(self,b=0,caption=None,position=0,real=0):
        dcolor=[(0,0,0),                      # black
                (255,255,255),                     # white
                (255,0,0),                     # red
                (0,255,0),                     # gree
                (0,0,255),                     # blue                
                ]
        if not real:                    # on filters that do not offer a preview, just do a return here
                                        # on other filters, you should apply the transform on a thumbnail and show it on self.wnd.thumbLabel
                                        
            #TODO: warn user that the preview is not broken. (that is, caption transform is not supposed to show a thumbnailed preview)

            #How do I remove the current pixmap of a label and then set it's caption ????
            #self.wnd.thumbLabel.setPixmap(QPixmap())
            #self.wnd.thumbLabel.clear()
            #self.wnd.thumbLabel.setCaption("No preview")            
            return
        # extract parameters
        caption=str(self.wnd.controls[self.wnd.textControls[0]][1].text())
        if not caption: caption="Sample caption"
        ifo=ImageFont.load(self.wnd.parent().imglibdir+"courB12.pil")
        #TODO: font is hardcoded, make it configurable later (since .pil font files must be used, make at least the size an option)
        image = Image.open(self.wnd.curfidname)
        draw = ImageDraw.Draw(image)
        position=int(self.wnd.controls[self.wnd.comboControls[0]][1].currentItem())
        scolor=dcolor[int(self.wnd.controls[self.wnd.comboControls[1]][1].currentItem())]
        tsz=draw.textsize(caption, font=ifo)
        imsz=image.size
        if position==0:                 # top keft
            draw.text((0,0),caption,fill=scolor,font=ifo)
        elif position==1:                 # top right
            draw.text((imsz[0]-4-tsz[0],0),caption,fill=scolor,font=ifo)
        elif position==2:               # bottom left
            draw.text((0,imsz[1]-4-tsz[1]),caption,fill=scolor,font=ifo)
        elif position==3:               # bottom riht
            draw.text((imsz[0]-4-tsz[0],imsz[1]-4-tsz[1]),caption,fill=scolor,font=ifo)
        image.save(real)
        del image
        del draw        
    def color(self,b=0,a0=10,real=0):
        if not real:
            image = Image.open(self.wnd.curdb.getThumbName(self.wnd.curfidname))
        else:
            image = Image.open(self.wnd.curfidname)
        enhancer = ImageEnhance.Color(image)
        if not real:
            enhancer.enhance(a0/100.0).save(".temp", "PPM")
            self.tPixmap.load(".temp")
            self.wnd.thumbLabel.setPixmap(self.tPixmap)
        else:
            enhancer.enhance(a0/100.0).save(real)
        del image
        del enhancer
    def bright(self,b=0,a0=10,real=0):
        if not real:
            image = Image.open(self.wnd.curdb.getThumbName(self.wnd.curfidname))
        else:
            image = Image.open(self.wnd.curfidname)
        enhancer = ImageEnhance.Brightness(image)
        if not real:        
            enhancer.enhance((a0)/100.0).save(".temp", "PPM")
            self.tPixmap.load(".temp")
            self.wnd.thumbLabel.setPixmap(self.tPixmap)
        else:
            enhancer.enhance(a0/100.0).save(real)  
        del image
        del enhancer
    def rotate(self,b=0,a0=10,real=0):
        if not real:
            image = Image.open(self.wnd.curdb.getThumbName(self.wnd.curfidname))
        else:
            image = Image.open(self.wnd.curfidname)
        image=image.rotate(int(a0)%360)
        if not real:        
            image.save(".temp", "PPM")
            self.tPixmap.load(".temp")
            self.wnd.thumbLabel.setPixmap(self.tPixmap)
        else:
            image.save(real)  
        del image
    def contrast(self,b=0,a0=10,real=0):
        if not real:
            image = Image.open(self.wnd.curdb.getThumbName(self.wnd.curfidname))
        else:
            image = Image.open(self.wnd.curfidname)
        enhancer = ImageEnhance.Contrast(image)
        if not real:        
            enhancer.enhance(a0/100.0).save(".temp", "PPM")
            self.tPixmap.load(".temp")
            self.wnd.thumbLabel.setPixmap(self.tPixmap)
        else:
            enhancer.enhance(a0/100.0).save(real)
        del image
        del enhancer
    def sharp(self,b=0,a0=10,real=0):
        if not real:
            image = Image.open(self.wnd.curdb.getThumbName(self.wnd.curfidname))
        else:
            image = Image.open(self.wnd.curfidname)
        enhancer = ImageEnhance.Sharpness(image)
        if not real:        
            enhancer.enhance(a0/100.0).save(".temp", "PPM")
            self.tPixmap.load(".temp")
            self.wnd.thumbLabel.setPixmap(self.tPixmap)
        else:
            enhancer.enhance(a0/100.0).save(real)  
        del image
        del enhancer
    def blur(self,b=0,a0=50,real=0):
        if not real:
            image = Image.open(self.wnd.curdb.getThumbName(self.wnd.curfidname))
            image.filter(ImageFilter.BLUR).save(".temp", "PPM")
            self.tPixmap.load(".temp")
            self.wnd.thumbLabel.setPixmap(self.tPixmap)            
        else:
            image = Image.open(self.wnd.curfidname)
            image.filter(ImageFilter.BLUR).save(real)
        del image
    def CONTOUR(self,b=0,a0=50,real=0):
        if not real:
            image = Image.open(self.wnd.curdb.getThumbName(self.wnd.curfidname))
            image.filter(ImageFilter.CONTOUR).save(".temp", "PPM")
            self.tPixmap.load(".temp")
            self.wnd.thumbLabel.setPixmap(self.tPixmap)            
        else:
            image = Image.open(self.wnd.curfidname)
            image.filter(ImageFilter.CONTOUR).save(real)
        del image
    def DETAIL(self,b=0,a0=50,real=0):
        if not real:
            image = Image.open(self.wnd.curdb.getThumbName(self.wnd.curfidname))
            image.filter(ImageFilter.DETAIL).save(".temp", "PPM")
            self.tPixmap.load(".temp")
            self.wnd.thumbLabel.setPixmap(self.tPixmap)            
        else:
            image = Image.open(self.wnd.curfidname)
            image.filter(ImageFilter.DETAIL).save(real)
        del image
    def EDGE_ENHANCE(self,b=0,a0=50,real=0):
        if not real:
            image = Image.open(self.wnd.curdb.getThumbName(self.wnd.curfidname))
            image.filter(ImageFilter.EDGE_ENHANCE).save(".temp", "PPM")
            self.tPixmap.load(".temp")
            self.wnd.thumbLabel.setPixmap(self.tPixmap)            
        else:
            image = Image.open(self.wnd.curfidname)
            image.filter(ImageFilter.EDGE_ENHANCE).save(real)
        del image
    def EDGE_ENHANCE_MORE(self,b=0,a0=50,real=0):
        if not real:
            image = Image.open(self.wnd.curdb.getThumbName(self.wnd.curfidname))
            image.filter(ImageFilter.EDGE_ENHANCE_MORE).save(".temp", "PPM")
            self.tPixmap.load(".temp")
            self.wnd.thumbLabel.setPixmap(self.tPixmap)            
        else:
            image = Image.open(self.wnd.curfidname)
            image.filter(ImageFilter.EDGE_ENHANCE_MORE).save(real)
        del image
    def EMBOSS(self,b=0,a0=50,real=0):
        if not real:
            image = Image.open(self.wnd.curdb.getThumbName(self.wnd.curfidname))
            image.filter(ImageFilter.EMBOSS).save(".temp", "PPM")
            self.tPixmap.load(".temp")
            self.wnd.thumbLabel.setPixmap(self.tPixmap)            
        else:
            image = Image.open(self.wnd.curfidname)
            image.filter(ImageFilter.EMBOSS).save(real)
        del image
    def FIND_EDGES(self,b=0,a0=50,real=0):
        if not real:
            image = Image.open(self.wnd.curdb.getThumbName(self.wnd.curfidname))
            image.filter(ImageFilter.FIND_EDGES).save(".temp", "PPM")
            self.tPixmap.load(".temp")
            self.wnd.thumbLabel.setPixmap(self.tPixmap)            
        else:
            image = Image.open(self.wnd.curfidname)
            image.filter(ImageFilter.FIND_EDGES).save(real)
        del image
    def invert(self,b=0,a0=50,real=0):
        if not real:
            image = Image.open(self.wnd.curdb.getThumbName(self.wnd.curfidname))
            ImageChops.invert(image).save(".temp", "PPM")
            self.tPixmap.load(".temp")
            self.wnd.thumbLabel.setPixmap(self.tPixmap)            
        else:
            image = Image.open(self.wnd.curfidname)
            ImageChops.invert(image).save(real)
        del image
    def SMOOTH(self,b=0,a0=50,real=0):
        if not real:
            image = Image.open(self.wnd.curdb.getThumbName(self.wnd.curfidname))
            image.filter(ImageFilter.SMOOTH).save(".temp", "PPM")
            self.tPixmap.load(".temp")
            self.wnd.thumbLabel.setPixmap(self.tPixmap)            
        else:
            image = Image.open(self.wnd.curfidname)
            image.filter(ImageFilter.SMOOTH).save(real)
        del image
    def SMOOTH_MORE(self,b=0,a0=50,real=0):
        if not real:
            image = Image.open(self.wnd.curdb.getThumbName(self.wnd.curfidname))
            image.filter(ImageFilter.SMOOTH_MORE).save(".temp", "PPM")
            self.tPixmap.load(".temp")
            self.wnd.thumbLabel.setPixmap(self.tPixmap)            
        else:
            image = Image.open(self.wnd.curfidname)
            image.filter(ImageFilter.SMOOTH_MORE).save(real)
        del image
    def SHARPEN(self,b=0,a0=50,real=0):
        if not real:
            image = Image.open(self.wnd.curdb.getThumbName(self.wnd.curfidname))
            image.filter(ImageFilter.SHARPEN).save(".temp", "PPM")
            self.tPixmap.load(".temp")
            self.wnd.thumbLabel.setPixmap(self.tPixmap)            
        else:
            image = Image.open(self.wnd.curfidname)
            image.filter(ImageFilter.SHARPEN).save(real)
        del image
        
class TransformWnd(TransformForm):
    def __init__(self, parent,ifile=None):
        self.curdb=Settings.curdb
        TransformForm.__init__(self, parent)
        self.imgIdx=0
        self.callback=None
        self.curfid=None
        self.curfidname=None
        self.curdb.cbs["Batch"].append(self.syncBatchCombo)
        self.curBatch=-1
        self.syncBatchCombo()        
        if ifile: # requested file to be shown immediately
            self.showFile(ifile)            
        else: #simple show first file on dbase
            self.showFile(self.curdb.meta.keys()[0])
        # Generic controls
        self.controls=[[self.slideLabel1,self.slider1,self.value1Edit],
                       [self.slideLabel2,self.slider2,self.value2Edit],
                       [self.slideLabel3,self.slider3,self.value3Edit],
                       [self.textLabel1,self.lineEdit1],
                       [self.comboLabel1,self.comboBox1],
                       [self.comboLabel2,self.comboBox2],
                       [self.checkBox1],
                       [self.checkBox2]
                       ]
        self.intControls=[0,1,2]        # index of the controls on self.controls that can control ints
        self.textControls=[3]
        self.checkControls=[6,7]
        self.comboControls=[4,5]
        
        self.initTransforms()
        self.saveMenu=QPopupMenu(self) 
        self.saveMenu.insertItem ( self.parent().globalslide.iconSet(),"&Preview with full size", self.onPreviewFull )
        self.saveMenu.insertSeparator()
        self.saveMenu.insertItem ( self.parent().fileSaveAction.iconSet(),"&Overwrite file", self.onSave )
        self.saveMenu.insertItem ( self.parent().fileSaveAsAction.iconSet(), "&Save as another file", self.onSaveAs )
       
    def initTransforms(self):
        self.transform=Transforms(self)
        self.transformList=self.transform.getTransforms()
        for t in self.transformList:
            self.transfCombo.insertItem(t[0])
        self.setupTransform(0)
    def setupTransform(self,t):
        """ hide controls according to transform being done. t is an index in the list of transforms returned by tf.getTransforms()
        also set callback
        """
        self.callback=self.transformList[t][1]

        for sli in self.controls:
            for cl in sli:
                cl.hide()
        intCount=0
        textCount=0
        comboCount=0
        for sl in self.transformList[t][2]:
            if sl[0]=="Int":
                for cl in self.controls[self.intControls[intCount]]: cl.show()
                self.controls[self.intControls[intCount]][0].setText(sl[1])
                intCount=intCount+1
            if sl[0]=="Text":
                for cl in self.controls[self.textControls[textCount]]: cl.show()
                self.controls[self.textControls[textCount]][0].setText(sl[1])
                textCount=textCount+1
            if sl[0]=="Combo":
                for cl in self.controls[self.comboControls[comboCount]]: cl.show()
                self.controls[self.comboControls[comboCount]][1].clear()
                for prm in sl[2]:
                    self.controls[self.comboControls[comboCount]][1].insertItem(prm)
                self.controls[self.comboControls[comboCount]][0].setText(sl[1])
                comboCount=comboCount+1
        self.callback()
    def showFile(self,fid):
        """show thumbnail of a file

        @param fid: file id
        """
        if not fid: return
        if type(fid)==int:              # show db image
            self.imgIdx=self.curdb.img.keys().index(fid)
            self.curfid=fid
            self.curfidname=self.curdb.img[fid][0]
            self.indexLabel.setText(str(self.imgIdx+1)+" / "+str(len(self.curdb.img.keys())))            
        else:                           # show system image
            self.imgIdx=0
            self.curfid=0
            self.curfidname=fid
            self.indexLabel.setText(fid)
        self.thumbLabel.setPixmap(QPixmap(self.curdb.getThumb(self.curfidname)))        
        if self.callback:self.callback()
    def previousBtn_clicked(self):
        self.imgIdx=self.imgIdx-1
        if self.imgIdx<0:self.imgIdx=len(self.curdb.img.keys()) -1
        self.showFile(self.curdb.img.keys()[self.imgIdx])
    def nextButton_clicked(self):
        self.imgIdx=self.imgIdx+1
        if self.imgIdx>=len(self.curdb.img.keys()):
            self.imgIdx=0
        self.showFile(self.curdb.img.keys()[self.imgIdx])
    def syncBatchCombo(self,what=None):
        """call it to sync curdb batch list to what the ui shows """
        self.batchcombo.clear()
        for bid in self.curdb.batches.keys():
            self.batchcombo.insertItem(self.curdb.batches[bid][0])
    def onSave(self):
        filename=self.curdb.img[self.curfid][0]

        (base,ext) = os.path.splitext(filename)
        fext=ext[1:].lower()

        origfname=filename[:]
        if fext in Settings.magick_ext:
            imgdb.convert(self.curfidname,Settings.curdb.thdir2+".cache.bmp")
            filename=Settings.curdb.thdir2+".cache.bmp"

        self.callback(a0=int(self.slider1.value()),real=filename)
        if fext in Settings.magick_ext:
            filename=origfname[:]
            imgdb.convert(Settings.curdb.thdir2+".cache.bmp",filename)
        
        QMessageBox.information( self, "imgSeek",filename+" saved.")
    def onSaveAs(self):
        filename=str(QFileDialog.getSaveFileName("", "*.*", self, "Choose filename for the new image."))
        if not filename: return
        
        (base,ext) = os.path.splitext(filename)
        fext=ext[1:].lower()

        origfname=filename[:]
        if fext in Settings.magick_ext:
            imgdb.convert(self.curfidname,Settings.curdb.thdir2+".cache.bmp")
            filename=Settings.curdb.thdir2+".cache.bmp"

        self.callback(a0=int(self.slider1.value()),real=filename)
        if fext in Settings.magick_ext:
            filename=origfname[:]
            imgdb.convert(Settings.curdb.thdir2+".cache.bmp",filename)
        
        QMessageBox.information( self, "imgSeek",filename+" saved.")
    def ApplyBtn_clicked(self):
        if self.targettab.currentPageIndex() == 0: # image
            self.saveMenu.popup(self.ApplyBtn.mapToGlobal(QPoint(0,0)))
        else:                           # batch
            flist=[]                    # list of filenames to change
            bid=self.curdb.batches.keys()[self.batchcombo.currentItem()]
            self.curdb.crawlBatchForFile(bid,flist)
            if not flist:
                QMessageBox.information( self, "imgSeek","This batch has no images in it.")
                return
            res=QMessageBox.information( self, "imgSeek","You are about to apply this Transform to %d image(s), are you sure ?"%len(flist),   QString("A&pply"), QString("Cancel"))
            if res: return
            for fil in flist:
                try:                
                    self.curfidname=fil
                    self.callback(a0=int(self.slider1.value()),real=fil)
                except:
                    traceback.print_exc()
                    QMessageBox.information( self, "imgSeek","Error applying transform to batch.")
            QMessageBox.information( self, "imgSeek","Transform applied to batch.")                    
    def onPreviewFull(self):
        fname=self.parent().dbdir+".cacheprev.bmp"
        #TODO: should I delete this cacheprev.bmp after usage ?
        self.callback(real=fname)
        ds=SlideShowDialog(self)
        ds.showImg(fname)        
    def targettab_selected(self,a0):
        pass
    def targettab_currentChanged(self,a0):
        if self.targettab.currentPageIndex() == 0: # image
            self.thumbLabel.show()
            self.showFile(self.curfid)
        else:                           # batch
            self.thumbLabel.hide()            
    def showbatcheditor_clicked(self):
        wnd=self.parent()
        wnd.editcategaction_activated()
    def batchcombo_activated(self,a0):
        pass
    def refreshbtn_clicked(self):
        res=QMessageBox.information( self, "imgSeek","Are you sure you want to revert to saved image ?",   QString("&Revert to what is saved"), QString("Cancel"))
        if res: return                    
        self.showFile(self.curfid)
    def hideEvent(self,e):
        return
##         if not self.dirty:
##             metaform.hideEvent(self,e)
##             return
##         res=QMessageBox.information( self, "imgSeek","Do you want to save changes made to current image ?",   QString("&Save"), QString("&Discard"))
##         metaform.hideEvent(self,e)
##         if res: return
##         self.saveMeta()
        
    def slider1_valueChanged(self,a0):
        self.value1Edit.setText(str(a0))
        self.callback(0,a0)
    def slider2_valueChanged(self,a0):
        self.value2Edit.setText(str(a0))        
        self.callback(1,a0)
    def slider3_valueChanged(self,a0):
        self.value3Edit.setText(str(a0))        
        self.callback(2,a0)
    def value2Edit_textChanged(self,a0):
        try:
            self.slider2.setValue(int(str(a0)))
        except:
            pass

    def value1Edit_textChanged(self,a0):
        try:
            self.slider1.setValue(int(str(a0)))
        except:
            pass

    def value3Edit_textChanged(self,a0):
        try:
            self.slider3.setValue(int(str(a0)))
        except:
            pass
    def transfCombo_activated(self,a0):
        self.setupTransform(a0)
